
local AddonData = Inspect.Addon.Detail("CarnageBook")
CB = AddonData.data

CB.Carnages = {}
CB.Lang = Inspect.System.Language()

CB.ZoneIDs = {
	z698CB7B72B3D69E9 = "Cape Jule",
	z754553DD46F46371 = "City Core",
	z48530386ED2EA5AD = "Eastern Holdings",
	z563CB77E4A32233F = "Ardent Domain",
	z4D8820D7EF52685C = "Kingsward",
	z2F1E4708BEC6A608 = "Ashora",
	z1C938C07F41C83CC = "Pelladane",
	z59124F7DD7F15825 = "Seratos",
	z39095BA75AD7DC03 = "Morban",
	z2F9C9E1FF91F9293 = "Steppes of Infinity",
}

CB.ZoneID_ZoneName = {}
CB.ZoneName_ZoneID = {}

if CB.Lang ~= "English" and CB.Lang ~= "French" and CB.Lang ~= "German" then
	CB.Lang = "English"
end

function SlashHandler(args)
	if string.len(args) == 0 then
		local player = Inspect.Unit.Detail('player')
		if player and player.zone and CB.ZoneID_ZoneName[player.zone] then
			args = CB.ZoneID_ZoneName[player.zone]
		end
		if string.len(args) == 0 then
			print("No carnages found for current zone.")
			return
		end
	end

	local print_completed = false
	local print_started = false
	local print_missing = true

	complete = {}
	current = {}

	zid = CB.ZoneName_ZoneID[args]

	if not zid then
		zid = CB.ZoneName_ZoneID[string.lower(args)]
	end

	if zid and CB.Carnages[CB.ZoneIDs[zid]] then
		for id, _ in pairs(Inspect.Quest.Complete()) do
			idlen = string.len(id)
			shortid = string.format("q%sxxxxxxxx", string.sub(id, idlen-15, idlen-8))
			complete[id] = true
			complete[shortid] = true
		end

		for id, _ in pairs(Inspect.Quest.List()) do
			idlen = string.len(id)
			shortid = string.format("q%sxxxxxxxx", string.sub(id, idlen-15, idlen-8))
			current[id] = true
			current[shortid] = true
		end

		print(string.format("Printing out missing carnages for '%s'", args))

		local missedtotal = 0
		local currenttotal = 0
		local completedtotal = 0

		for id, quest in pairs(CB.Carnages[CB.ZoneIDs[zid]]) do
			shortid = string.format("q%sxxxxxxxx", string.sub(id, 2, 9))
			longid = string.format("qFFFFFFFF%sxxxxxxxx", string.sub(id, 2, 9))
			if complete[id] or complete[shortid] or complete[longid] then
				if print_completed then
					print(string.format("Completed - %s", quest[CB.Lang].Name))
				end
				completedtotal = completedtotal + 1
			elseif current[id] or current[shortid] or current[longid] then
				if print_started then
					print(string.format("Started      - %s", quest[CB.Lang].Name))
				end
				currenttotal = currenttotal + 1
			else
				if print_missing then
					print(string.format("  %s (%s)", quest[CB.Lang].Name, quest[CB.Lang].Summary))
				end
				missedtotal = missedtotal + 1
			end
		end
		print(string.format("Total: %d, Completed: %d, Current: %d, Missing: %d", (completedtotal+currenttotal+missedtotal), completedtotal, currenttotal, missedtotal))
	else
		print(string.format("No zone found (%s)", args))
	end
end

function InitZones(ModID)
	if ModID == "CarnageBook" then
		for z, name in pairs(CB.ZoneIDs) do
			ztmp = Inspect.Zone.Detail(z)
			if ztmp and ztmp.name then
				CB.ZoneID_ZoneName[z] = ztmp.name
				CB.ZoneName_ZoneID[ztmp.name] = z
				CB.ZoneName_ZoneID[string.lower(ztmp.name)] = z
			else
				CB.ZoneID_ZoneName[z] = name
			end
			CB.ZoneName_ZoneID[name] = z
			CB.ZoneName_ZoneID[string.lower(name)] = z
		end
		print("Carnage Book "..AddonData.toc.Version.." Loaded.")
	end
end

table.insert(Event.Addon.Load.End, {InitZones, "CarnageBook", "Init zone tables"})
table.insert(Command.Slash.Register("carnage"), {SlashHandler, "CarnageBook", "Slash Command"})
