local toc, data = ...
local AddonId = toc.identifier

-- Local config options -----------------------------------------------
local mainWidth = 160
local mainHeight = 60
local innerBarWidth = mainWidth * 0.9
local innerBarHeight = mainHeight * (1 / 15)
local mainBarHeight = mainHeight / 3
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local DoubleBarLeft = WT.UnitFrame:Template("DoubleBarLeft")
DoubleBarLeft.Configuration.Name = "Double Bar Unit Frame / Fades Left"
DoubleBarLeft.Configuration.RaidSuitable = false
DoubleBarLeft.Configuration.FrameType = "Frame"
DoubleBarLeft.Configuration.Width = mainWidth
DoubleBarLeft.Configuration.Height = mainHeight
DoubleBarLeft.Configuration.Resizable = { mainWidth, mainHeight, 800, 300 }
---------------------------------------------------------------------------------

-- Override the buff filter to hide some buffs ----------------------------------
local buffPriorities = 
{
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
}
function DoubleBarLeft:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end
---------------------------------------------------------------------------------

function DoubleBarLeft:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				--width=backW,  
				alpha=1, color={r=0,g=0,b=0,a=0}
			},
			{
				-- Generic Element Configuration
				id="barBackground", type="Bar", parent="frameBackdrop", layer=10,
				attach = { 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT" },
				},
				binding="doubleBackground", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/FadedBoxRight.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="frameHealthColor", type="Frame", parent="frameBackdrop", layer=15, alpha=0.5,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={0, 1/3} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={1, 2/3} },
				},
				colorBinding="callingColor", 
			},
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frameBackdrop", layer=20, 
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={0, 1/3} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={1, 2/3} },
				},
				--height=mainBarHeight,
				growthDirection="right",
				binding="healthPercent", colorBinding="callingColor", 
				texAddon=AddonId, texFile="img/DoubleBarTexture.png",
				backgroundColor={r=0, g=0, b=0, a=0.5},
			},
			{
				-- Generic Element Configuration
				id="frameResourceColor", type="Frame", parent="frameBackdrop", layer=25, alpha=0.5,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={1/20, 7/15} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={19/20, 8/15} },
				},
				colorBinding="resourceColor", 
			},
			{
				-- Generic Element Configuration
				id="barResourceColor", type="Bar", parent="frameBackdrop", layer=30,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={1/20, 7/15} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={19/20, 8/15} },
				},
				growthDirection="right",
				binding="resourcePercent", colorBinding="resourceColor", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0.5}
			},
			{
				-- Generic Element Configuration
				id="barHealthBorder", type="Bar", parent="frameBackdrop", layer=30, 
				attach = { 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT" },
				},
				-- Type Specific Element Configuration
				binding="doubleBackground",
				texAddon=AddonId, texFile="img/DoubleBarBackground.png", 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=40, alpha=1,
				attach = {{ point="TOPCENTER", element="frame", targetPoint={0.5, 1/20} }},
				visibilityBinding="name", 
				-- Type Specific Element Configuration
				text="{name}", default="", color={r=1,g=1,b=1,a=1}, backgroundColor={r=0, g=0, b=0, a=1},
				linkedHeightElement="barHealth", linkedHeightScale=0.60, maxLength=25,
			},
			{
				-- Generic Element Configuration
				id="labelMark", type="Label", parent="frame", layer=40, alpha=1,
				attach = {
					{ point="TOP", element="frame", targetPoint="TOP" },
					{ point="RIGHT", element="frame", targetPoint="RIGHT" },
				},
				visibilityBinding="mark",
				-- Type Specific Element Configuration
				text="{mark}", default="X",
				linkedHeightElement="frame", linkedHeightScale=1/4, maxLength=2,
			},
			{
				-- Generic Element Configuration
				id="imgCombat", type="Image", parent="frame", layer=60,
				attach = {
					{ point={5/12,7/12}, element="barHealth", targetPoint="BOTTOMLEFT" },
					{ point={5/6,1/6}, element="barResourceColor", targetPoint="BOTTOMLEFT" },
				}, 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/combat.png", visibilityBinding="combat",
			},
		}
	}
	
	for idx,element in ipairs(template.elements) do
		if options.excludeBuffs and element.type=="BuffPanel" then
			-- Don't add buff panels if excluding them
		else
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end


-- Frame Configuration Options --------------------------------------------------
local DoubleBarRight = WT.UnitFrame:Template("DoubleBarRight")
DoubleBarRight.Configuration.Name = "Double Bar Unit Frame / Fades Right"
DoubleBarRight.Configuration.RaidSuitable = false
DoubleBarRight.Configuration.FrameType = "Frame"
DoubleBarRight.Configuration.Width = mainWidth
DoubleBarRight.Configuration.Height = mainHeight
DoubleBarRight.Configuration.Resizable = { mainWidth, mainHeight, 800, 300 }
---------------------------------------------------------------------------------

-- Override the buff filter to hide some buffs ----------------------------------
function DoubleBarRight:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end
---------------------------------------------------------------------------------

function DoubleBarRight:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" },
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				--width=backW,  
				alpha=1, color={r=0,g=0,b=0,a=0}
			},
			{
				-- Generic Element Configuration
				id="barBackground", type="Bar", parent="frameBackdrop", layer=10,
				attach = { 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT" },
				},
				binding="doubleBackground", 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/FadedBoxLeft.png", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="frameHealthColor", type="Frame", parent="frameBackdrop", layer=15, alpha=0.5,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={0, 1/3} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={1, 2/3} },
				},
				colorBinding="callingColor", 
			},
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frameBackdrop", layer=20, 
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={0, 1/3} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={1, 2/3} },
				},
				--height=mainBarHeight,
				growthDirection="left",
				binding="healthPercent", colorBinding="callingColor", 
				texAddon=AddonId, texFile="img/DoubleBarTexture.png",
				backgroundColor={r=0, g=0, b=0, a=0.5},
			},
			{
				-- Generic Element Configuration
				id="frameResourceColor", type="Frame", parent="frameBackdrop", layer=25, alpha=0.5,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={1/20, 7/15} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={19/20, 8/15} },
				},
				colorBinding="resourceColor", 
			},
			{
				-- Generic Element Configuration
				id="barResourceColor", type="Bar", parent="frameBackdrop", layer=30,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint={1/20, 7/15} },
					{ point="BOTTOMRIGHT", element="frame", targetPoint={19/20, 8/15} },
				},
				growthDirection="left",
				binding="resourcePercent", colorBinding="resourceColor", alpha=1,
				backgroundColor={r=0, g=0, b=0, a=0.5}
			},
			{
				-- Generic Element Configuration
				id="barHealthBorder", type="Bar", parent="frameBackdrop", layer=30, 
				attach = { 
					{ point="TOPLEFT", element="frameBackdrop", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT" },
				},
				-- Type Specific Element Configuration
				binding="doubleBackground",
				texAddon=AddonId, texFile="img/DoubleBarBackground.png", 
				backgroundColor={r=0, g=0, b=0, a=0},
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=40, alpha=1,
				attach = {{ point="TOPCENTER", element="frame", targetPoint={0.5, 1/20} }},
				visibilityBinding="name", 
				-- Type Specific Element Configuration
				text="{name}", default="", color={r=1,g=1,b=1,a=1}, backgroundColor={r=0, g=0, b=0, a=1},
				linkedHeightElement="barHealth", linkedHeightScale=0.60, maxLength=25,
			},
			{
				-- Generic Element Configuration
				id="labelMark", type="Label", parent="frame", layer=40, alpha=1,
				attach = {
					{ point="TOP", element="frame", targetPoint="TOP" },
					{ point="LEFT", element="frame", targetPoint="LEFT" },
				},
				visibilityBinding="mark",
				-- Type Specific Element Configuration
				text="{mark}", default="X",
				linkedHeightElement="frame", linkedHeightScale=1/4, maxLength=2,
			},
			{
				-- Generic Element Configuration
				id="imgCombat", type="Image", parent="frame", layer=60,
				attach = {
					{ point={7/12,7/12}, element="barHealth", targetPoint="BOTTOMRIGHT" },
					{ point={1/6,1/6}, element="barResourceColor", targetPoint="BOTTOMRIGHT" },
				}, 
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/combat.png", visibilityBinding="combat",
			},
		}
	}
	
	for idx,element in ipairs(template.elements) do
		if options.excludeBuffs and element.type=="BuffPanel" then
			-- Don't add buff panels if excluding them
		else
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end


WT.Unit.CreateVirtualProperty("doubleBackground", { "name" },
	function(unit)
		if unit.name then
			return 100
		else
			return 0
		end
	end)
