local toc, data = ...
local AddonId = toc.identifier

-- Local config options ---------------------------------------------------------
local scale = 0.5
local healthBarWidth = 128 * scale
local healthBarHeight = 256 * scale
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local HealthBar = WT.UnitFrame:Template("HealthBar")
HealthBar.Configuration.Name = "Health Bar Unit Frame"
HealthBar.Configuration.RaidSuitable = false
HealthBar.Configuration.FrameType = "Frame"
HealthBar.Configuration.Width = healthBarWidth
HealthBar.Configuration.Height = healthBarHeight
HealthBar.Configuration.Resizable = { healthBarWidth, healthBarHeight, 256, 512 }
---------------------------------------------------------------------------------


function HealthBar:Construct(options)

	local template =
	{
		elements = 
		{
			{
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				}, 				
				visibilityBinding="id",
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.5,
				color={r=0,g=0,b=0,a=0.3},
			},
			{
				id="barHealth", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" },
				},
				growthDirection="up",
				binding="healthPercent", colorBinding="callingColor",
				texAddon=AddonId, texFile="img/WinBrH.png", 
				backgroundColor={r=0, g=0, b=0, a=0.7}
			},
			{
				id="frameBlocked", type="Frame", parent="frameBackdrop", layer=15, alpha=0.3,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				},
				colorBinding="callingColor", 
			},
			{
				id="labelHealthPercent", type="Label", parent="frame", layer=14,
				attach = 
				{ 
					{ point="CENTER", element="frame", targetPoint="CENTER" },
				}, visibilityBinding="id", text="{healthPercent}%",fontSize=24,
				color={r=1,g=1,b=1,a=1},
			}, 
			{
				id="labelHealthMax", type="Label", parent="frame", layer=18,
				attach = 
				{ 
					{ point="BOTTOMCENTER", element="barHealth", targetPoint="BOTTOMCENTER", offsetX=0, offsetY=0 },
				}, visibilityBinding="id", text="{healthMax}",fontSize=24,
				color={r=1,g=1,b=1,a=1},
			},
			{
				id="labelHealth", type="Label", parent="frame", layer=18,
				attach = 
				{ 
					{ point="TOPCENTER", element="barHealth", targetPoint="TOPCENTER", offsetX=0, offsetY=0 },
				}, visibilityBinding="id", text="{health}",fontSize=24,
				color={r=1,g=1,b=1,a=1},
			}, 
		}
	}
	
	for idx,element in ipairs(template.elements) do
		self:CreateElement(element) 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
	
end
