local toc, data = ...
local AddonId = toc.identifier

-- Local config options ---------------------------------------------------------
local scale = 0.5
local manaBarWidth = 128 * scale
local manaBarHeight = 256 * scale
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local ManaBar = WT.UnitFrame:Template("ManaBar")
ManaBar.Configuration.Name = "Mana Bar Unit Frame"
ManaBar.Configuration.RaidSuitable = false
ManaBar.Configuration.FrameType = "Frame"
ManaBar.Configuration.Width = manaBarWidth
ManaBar.Configuration.Height = manaBarHeight
ManaBar.Configuration.Resizable = { manaBarWidth, manaBarHeight, 256, 512 }
---------------------------------------------------------------------------------


function ManaBar:Construct(options)

	local template =
	{
		elements = 
		{
			{
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				}, 				
				visibilityBinding="id",
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.5,
				color={r=0,g=0,b=0,a=0.3},
			},
			{
				id="barResource", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" },
				},
				growthDirection="up",
				binding="resourcePercent", colorBinding="resourceColor",
				texAddon=AddonId, texFile="img/WinBrH.png", 
				backgroundColor={r=0, g=0, b=0, a=0.7}
			},
			{
				id="frameBlocked", type="Frame", parent="frameBackdrop", layer=15, alpha=0.3,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				},
				colorBinding="resourceColor", 
			},
			{
				id="labelResourcePercent", type="Label", parent="frame", layer=14,
				attach = 
				{ 
					{ point="CENTER", element="barResource", targetPoint="CENTER" },
				}, visibilityBinding="id", text="{resourcePercent}%",fontSize=24,
				color={r=1,g=1,b=1,a=1},
			}, 
			{
				id="labelResourceMax", type="Label", parent="frame", layer=18,
				attach = 
				{ 
					{ point="BOTTOMCENTER", element="barResource", targetPoint="BOTTOMCENTER" },
				}, visibilityBinding="id", text="{resourceMax}",fontSize=24,
				color={r=1,g=1,b=1,a=1},
			},
			{
				id="labelResource", type="Label", parent="frame", layer=18,
				attach = 
				{ 
					{ point="TOPCENTER", element="barResource", targetPoint="TOPCENTER" },
				}, visibilityBinding="id", text="{resource}",fontSize=24,
				color={r=1,g=1,b=1,a=1},
			}, 
		}
	}
	
	for idx,element in ipairs(template.elements) do
		self:CreateElement(element) 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
	
	self.ConfigurationDialog = configDialog
end
