nkTools = {}
nkTools.table = {}
nkTools.math = {}
nkTools.settings = {}
nkTools.various = {}
nkTools.lang = {}

local privateVars = { curLanguage = Inspect.System.Language() }

-- table handling

function nkTools.table.isMember (checkTable, element)
  
  for idx, value in pairs(checkTable) do
    if value == element then return true end
  end
  
  return false
  
end

function nkTools.table.getTablePos (checkTable, element)

	for idx, value in pairs (checkTable) do
		if value == element then return idx end
	end
	
	return -1

end

function nkTools.table.getSortedKeys (tableData)

	local tempTable = {}
    
    for k, data in pairs(tableData) do 
        table.insert(tempTable, k) 
    end
	
	table.sort(tempTable, function (a, b) return string.lower(a) < string.lower(b) end)
	return tempTable
	
end

function nkTools.table.merge (table1, table2)

	for k, v in pairs (table2) do
		table.insert (table1, v)
	end

end

function nkTools.table.mergeIndexed (table1, table2)

	for k, v in pairs (table2) do
		table1[k] = v
	end

end

function nkTools.table.getKeyByValue (tableData, value)

	for k, v in pairs (tableData) do
		if v == value then return k end
	end
	
	return nil

end

function nkTools.table.copy (tableToCopy)

    local lookup_table = {}
    local function _copy(tableToCopy)
        if type(tableToCopy) ~= "table" then
            return tableToCopy
        elseif lookup_table[tableToCopy] then
            return lookup_table[tableToCopy]
        end
        local new_table = {}
        lookup_table[tableToCopy] = new_table
        for index, value in pairs(tableToCopy) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(tableToCopy))
    end
    return _copy(tableToCopy)
end

-- string handling

function string:split(delimiter)
  
  local result = { }
  local from = 1

  local delim_from, delim_to = string.find( self, delimiter, from )
  
  while delim_from do
    table.insert( result, string.sub( self, from , delim_from-1 ) )
    from = delim_to + 1
    delim_from, delim_to = string.find( self, delimiter, from )
  end
  table.insert( result, string.sub( self, from ) )
  return result
  
end

function string:strRight (value)

	local pos, len = string.find ( self, value)
	if pos == nil then return value end
	
	pos = pos + len
	return string.sub ( self, pos)

end

function string:strRightRegEx (value)

	local pos, len = string.find ( self, value)
	if pos == nil then return value end
	
	pos = pos + len
	return string.sub ( self, pos)

end

function string:strLeft (value)

	local pos = string.find ( self, value)
	return string.sub ( self, 1, pos)

end

-- math functions

function nkTools.math.round (num, idp)

	if num == nil then return nil end

	local mult = 10^(idp or 0)
	return math.floor(tonumber(num) * mult + 0.5) / mult
end

-- various functions

function nkTools.various.convertSeconds (seconds)

	if seconds == nil then return nil end

	if seconds <= 60 then return string.format ('%dS', seconds) end
	local minutes = seconds / 60
	return string.format ('%dM', nkTools.math.round(minutes, 0))

end

function nkTools.various.getSystemVersion ()

	local version = Inspect.System.Version()
	local tempArray = version.internal:split('-')
	
	return tempArray[2]

end

-- settings handling

function nkTools.settings.checkSettings (savedVariable, defaultValues, path)

	if savedVariable == nil then
		if type(defaultValues) == 'table' then
			return nkTools.table.copy(defaultValues)
		else
			return defaultValues
		end
	end
	
	if type(defaultValues) == 'table' then
		local newTable = nkTools.table.copy(savedVariable)
		
		for k, v in pairs (defaultValues) do
			if path ~= nil then print (string.format('%s.%s', path, k)) end
			
			if newTable[k] == nil then
				if type(v) == 'table' then
					if path ~= nil then				
						newTable[k] = nkTools.settings.checkSettings (newTable[k], v, string.format('%s.%s', path, k))
					else
						newTable[k] = nkTools.settings.checkSettings (newTable[k], v)
					end
				else
					newTable[k] = v
				end
			elseif type(v) == 'table' then
				if path ~= nil then
					newTable[k] = nkTools.settings.checkSettings (newTable[k], v, string.format('%s.%s', path, k))
				else
					newTable[k] = nkTools.settings.checkSettings (newTable[k], v)
				end
			end
		end
		
		return newTable
	end

end

-- language handling

function nkTools.lang.getLanguage ()

	if nkToolsSetup == nil then
		return privateVars.curLanguage
	elseif nkToolsSetup.language == nil then 
		return privateVars.curLanguage
	else
		return nkToolsSetup.language
	end

end

function nkTools.lang.getLanguageShort ()

	if nkTools.lang.getLanguage() == 'German' then
		return 'DE'
	elseif nkTools.lang.getLanguage() == 'French' then		
		return 'FR'
	elseif nkTools.lang.getLanguage() == 'Russian' then		
		return 'RU'
	else
		return 'EN'
	end

end

function nkTools.lang.setLanguage (language) 

	if nkToolsSetup == nil then nkToolsSetup = {} end
	nkToolsSetup.language = language
	privateVars.curLanguage = language 

end

function nkTools.lang.resetLanguage()

	if nkToolsSetup == nil then nkToolsSetup = {} end
	nkToolsSetup.language = nil

end
